EMPTY_FTE = '0'


def transform_successfactor_data(bigquery_successfactor):
    data_json = {
        'office_name': 'Madrid - Ing',
        'FTE': 1,
    }

    for data in bigquery_successfactor:
        data_json['office_name'] = data.get('office', '')
        data_json['FTE'] = data.get('fte', 0)

    return data_json


def transform_successfactor_all_data(bigquery_successfactor):
    data_json = {}

    for data in bigquery_successfactor:
        email = data.get('email', '')
        hiringdate = data.get('hiringdate', '')
        enddate = data.get('internshipdate', '')

        if email not in data_json.keys():
            data_json[email] = {}

        fte = data.get('fte', EMPTY_FTE) or EMPTY_FTE  # For null values
        data_json[email]['supervisor'] = data.get('supervisor', '')
        data_json[email]['office_name'] = data.get('office', '')
        data_json[email]['department'] = data.get('department', '')
        data_json[email]['FTE'] = float(fte.replace(',', '.'))
        data_json[email]['hiring_date'] = hiringdate
        data_json[email]['enddate'] = enddate
        data_json[email]['name'] = data.get('name', '')

    return data_json


def transform_all_capacities(capacities):
    data_json = {}

    for data in capacities:
        email = data.get('email')
        if email not in data_json:
            data_json[email] = {'fte': data.get('fte')}

    return data_json


def group_task_by_email(month_data, group_by_full_date=False):
    data_by_email = {}
    for task in month_data:
        taskTimeSpent = task.get('timeSpent', 0)
        taskEmail = task.get('authorEmail')
        taskStarted = task.get('started')
        grouped_date = (
            taskStarted.strftime('%Y-%m-%d')
            if group_by_full_date
            else taskStarted.day
        )
        if taskEmail not in data_by_email.keys():
            data_by_email[taskEmail] = {'totalHours': 0, 'days': {}}

        if grouped_date not in data_by_email[taskEmail]['days']:
            data_by_email[taskEmail]['days'][grouped_date] = 0

        data_by_email[taskEmail]['totalHours'] += taskTimeSpent
        data_by_email[taskEmail]['days'][grouped_date] += taskTimeSpent

    return data_by_email


def group_task_by_outdated(month_data):
    data_by_email = {}
    for task in month_data:
        taskTimeSpent = task.get('timeSpent', 0)
        taskEmail = task.get('authorEmail')
        taskStarted = task.get('started').strftime('%Y-%m-%d')
        taskCreated = task.get('created').strftime('%Y-%m-%d')
        taskIssueKey = task.get('issueKey')

        if taskEmail not in data_by_email.keys():
            data_by_email[taskEmail] = {'totalHours': 0, 'days': {}}

        if taskCreated not in data_by_email[taskEmail]['days']:
            data_by_email[taskEmail]['days'][taskCreated] = {
                'total': 0,
                'tasks': {},
            }

        if (
            taskIssueKey
            not in data_by_email[taskEmail]['days'][taskCreated]['tasks']
        ):  # noqa: E501
            data_by_email[taskEmail]['days'][taskCreated]['tasks'][
                taskIssueKey
            ] = {}  # noqa: E501

        if (
            taskStarted
            not in data_by_email[taskEmail]['days'][taskCreated]['tasks'][
                taskIssueKey
            ]
        ):  # noqa: E501
            data_by_email[taskEmail]['days'][taskCreated]['tasks'][
                taskIssueKey
            ][
                taskStarted
            ] = 0  # noqa: E501

        data_by_email[taskEmail]['totalHours'] += taskTimeSpent
        data_by_email[taskEmail]['days'][taskCreated]['total'] += taskTimeSpent
        data_by_email[taskEmail]['days'][taskCreated]['tasks'][taskIssueKey][
            taskStarted
        ] += taskTimeSpent  # noqa: E501

    return data_by_email


def group_daily_tasks_by_email(month_data):
    data_by_email = {}
    for task in month_data:
        taskTimeSpent = task.get('timeSpent', 0)
        taskEmail = task.get('authorEmail')
        taskStarted = task.get('started')
        grouped_date = taskStarted.day

        if taskEmail not in data_by_email.keys():
            data_by_email[taskEmail] = {
                'totalHours': 0,
                'days': {},
            }

        if grouped_date not in data_by_email[taskEmail]['days']:
            data_by_email[taskEmail]['days'][grouped_date] = 0

        data_by_email[taskEmail]['totalHours'] += taskTimeSpent
        data_by_email[taskEmail]['days'][grouped_date] += taskTimeSpent

    return data_by_email



import datetime
import unittest

from ..dates import get_first_day, get_last_day


class TestSuite(unittest.TestCase):
    def test_get_first_day_should_return_first_date_from_a_date(self):
        year = 2022
        month = 2
        day = 28
        date = datetime.date(year, month, day)
        expected_result = datetime.date(year, month, 1)

        first_date = get_first_day(date=date)

        self.assertEqual(first_date, expected_result)

    def test_get_last_day_should_return_last_date_from_a_date(self):
        year = 2022
        month = 2
        day = 10
        date = datetime.date(year, month, day)
        expected_result = datetime.date(year, month, 28)

        last_day = get_last_day(date=date)

        self.assertEqual(last_day, expected_result)



import unittest
from datetime import date

from ms_imputedhours_core.employee import calculate_real_capacity

DEFAULT_AGREEMENT_HOURS = {
    'totalHours': 142.0,
    'days': {
        '1-4': '0',
        '2-4': '0',
        '3-4': '8',
        '4-4': '5.5',
        '5-4': '0',
        '6-4': '0',
        '7-4': '8',
        '8-4': '8',
        '9-4': '0',
        '10-4': '8',
        '11-4': '5.5',
        '12-4': '0',
        '13-4': '0',
        '14-4': '8',
        '15-4': '8',
        '16-4': '8',
        '17-4': '8',
        '18-4': '5.5',
        '19-4': '0',
        '20-4': '0',
        '21-4': '8',
        '22-4': '8',
        '23-4': '8',
        '24-4': '8',
        '25-4': '5.5',
        '26-4': '0',
        '27-4': '0',
        '28-4': '8',
        '29-4': '8',
        '30-4': '8',
    },
}


class TestRealCapacity(unittest.TestCase):
    def test_no_hiring_date(self):
        hiring_date = None
        end_date = None
        from_date = date(2023, 2, 1)
        to_date = date(2023, 2, 28)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 0)

    def test_same_month_hiring_date(self):
        hiring_date = date(2023, 4, 1)
        end_date = None
        from_date = date(2023, 4, 1)
        to_date = date(2023, 4, 30)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 142.0)

    def test_same_month_hiring_and_end_date(self):
        hiring_date = date(2023, 4, 1)
        end_date = date(2023, 4, 15)
        from_date = date(2023, 4, 1)
        to_date = date(2023, 4, 30)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 59.0)

    def test_same_month_end_date(self):
        hiring_date = date(2023, 3, 1)
        end_date = date(2023, 4, 15)
        from_date = date(2023, 5, 1)
        to_date = date(2023, 5, 31)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 0)

    def test_end_date_within_range(self):
        agreement_hours = {
            'totalHours': 142.0,
            'days': {
                '1-5': '0',
                '2-5': '0',
                '3-5': '8',
                '4-5': '5.5',
                '5-5': '0',
                '6-5': '0',
                '7-5': '8',
                '8-5': '8',
                '9-5': '0',
                '10-5': '8',
                '11-5': '5.5',
                '12-5': '0',
                '13-5': '0',
                '14-5': '8',
                '15-5': '8',
                '16-5': '8',
                '17-5': '8',
                '18-5': '5.5',
                '19-5': '0',
                '20-5': '0',
                '21-5': '8',
                '22-5': '8',
                '23-5': '8',
                '24-5': '8',
                '25-5': '5.5',
                '26-5': '0',
                '27-5': '0',
                '28-5': '8',
                '29-5': '8',
                '30-5': '8',
            },
        }
        hiring_date = date(2023, 3, 1)
        end_date = date(2023, 5, 15)
        from_date = date(2023, 5, 1)
        to_date = date(2023, 5, 31)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            agreement_hours,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 59)

    def test_start_date_after_end_date(self):
        hiring_date = date(2023, 1, 1)
        end_date = date(2023, 2, 15)
        from_date = date(2023, 5, 1)
        to_date = date(2023, 5, 31)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 0)

    def test_partial_month(self):
        hiring_date = date(2023, 3, 1)
        end_date = None
        from_date = date(2023, 4, 2)
        to_date = date(2023, 4, 30)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 142.0)

    def test_full_month(self):
        hiring_date = date(2023, 3, 1)
        end_date = None
        from_date = date(2023, 4, 1)
        to_date = date(2023, 4, 30)
        fte = 1
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 142.0)

    def test_fte(self):
        hiring_date = date(2023, 3, 1)
        end_date = None
        from_date = date(2023, 4, 1)
        to_date = date(2023, 4, 30)
        fte = 0.5
        calculate_range = True
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 71.0)

    def test_calculate_range_false(self):
        hiring_date = date(2023, 4, 15)
        end_date = None
        from_date = date(2023, 4, 1)
        to_date = date(2023, 4, 30)
        fte = 1
        calculate_range = False
        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, 142.0)



import unittest
from datetime import datetime
from unittest.mock import Mock

from freezegun import freeze_time

from ms_imputedhours_core.employee import calculate_real_capacity, get_real_fte

DEFAULT_AGREEMENT_HOURS = {  # January 2022
    'totalHours': 168.0,
    'days': {
        '1-1': '0',
        '2-1': '0',
        '3-1': '8',
        '4-1': '8',
        '5-1': '8',
        '6-1': '8',
        '7-1': '8',
        '8-1': '0',
        '9-1': '0',
        '10-1': '8',
        '11-1': '8',
        '12-1': '8',
        '13-1': '8',
        '14-1': '8',
        '15-1': '0',
        '16-1': '0',
        '17-1': '8',
        '18-1': '8',
        '19-1': '8',
        '20-1': '8',
        '21-1': '8',
        '22-1': '0',
        '23-1': '0',
        '24-1': '8',
        '25-1': '8',
        '26-1': '8',
        '27-1': '8',
        '28-1': '8',
        '29-1': '0',
        '30-1': '0',
        '31-1': '8',
    },
}


class TestSuite(unittest.TestCase):
    @freeze_time("2022-11-11")
    def test_get_real_fte_returns_successfactor_fte_when_date_is_not_future_and_employee_has_not_fte(
        self,
    ):  # noqa: E501
        expected_result = 1.0  # 1.0 === 100%
        successfactor_data = {'FTE': 100}
        employeeFTE = {}
        date = Mock()
        date.year = 2021
        date.month = 10

        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_get_real_fte_returns_employee_fte_when_date_is_not_future_and_employee_has_fte(
        self,
    ):  # noqa: E501
        expected_result = 0.5  # 0.5 === 50%
        successfactor_data = {'FTE': 100}
        employeeFTE = {'fte': 0.5}
        date = Mock()
        date.year = 2021
        date.month = 10

        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2021-11-11")
    def test_get_real_fte_returns_successfactor_fte_when_date_is_for_future(
        self,
    ):  # noqa: E501
        expected_result = 1  # 100%
        successfactor_data = {'FTE': 100}
        employeeFTE = {'fte': 50}
        date = Mock()
        date.year = 2022
        date.month = 10

        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_full_capacity_when_is_normal_month(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 168.0
        hiring_date = datetime.strptime('17/09/2021', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1.0  # 100%
        calculate_range = True

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_half_capacity_when_is_normal_month_and_fte_zero_point_five(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 84.0
        hiring_date = datetime.strptime('17/09/2021', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5  # 50%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_zero_when_date_is_before_hired(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 0.0
        hiring_date = datetime.strptime('01/02/2022', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1  # 100%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-1-31")
    def test_calculate_real_capacity_return_all_capacity_when_calculate_range_is_false(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 168.0
        hiring_date = datetime.strptime('01/02/2021', '%d/%m/%Y')
        end_date = datetime.strptime('15/01/2022', '%d/%m/%Y')
        from_date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1  # 100%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-1-31")
    def test_calculate_real_capacity_return_all_capacity_when_employee_end_or_hiring_date_months_are_not_equal_thant_calculation(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 168.0
        hiring_date = datetime.strptime('01/02/2021', '%d/%m/%Y')
        end_date = datetime.strptime('15/02/2022', '%d/%m/%Y')
        from_date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1  # 100%
        calculate_range = True

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-01-31")
    def test_calculate_real_capacity_return_only_work_days_when_employee_has_end_date(
        self,
    ):  # noqa: E501
        expected_result = 80.0
        hiring_date = datetime.strptime('01/02/2021', '%d/%m/%Y')
        end_date = datetime.strptime('15/01/2022', '%d/%m/%Y')
        from_date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1  # 100%
        calculate_range = True

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-01-31")
    def test_calculate_real_capacity_return_only_work_days_when_employee_hiring_date_is_same_month_than_calculation(  # noqa: E501
        self,
    ):
        expected_result = 88.0
        hiring_date = datetime.strptime('15/01/2022', '%d/%m/%Y')
        end_date = None
        from_date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1  # 100%
        calculate_range = True

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-01-31")
    def test_calculate_real_capacity_return_only_work_days_when_employee_hiring_and_end_date_are_in_calculation_month(  # noqa: E501
        self,
    ):
        expected_result = 80.0
        hiring_date = datetime.strptime('15/01/2022', '%d/%m/%Y')
        end_date = datetime.strptime('28/01/2022', '%d/%m/%Y')
        from_date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1  # 100%
        calculate_range = True

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            from_date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)



import os
from datetime import datetime

BIGQUERY_EMPLOYEE_CAPACITY = os.getenv(
    'BIGQUERY_EMPLOYEE_CAPACITY_TABLE',
    'ms--tiber-happeo--pro--aa82.jiradataintegration.employeecapacity',
)


class BigqueryUpdateQueryBuilder(object):
    def __init__(
        self,
        email,
        year,
        month,
        jira,
        fte,
        office,
        capacity,
        hiring_date,
        enddate,
        realcapacity,
    ):
        self.today = datetime.today()

        self.email = email
        self.year = year
        self.month = month
        self.jira = jira
        self.fte = fte
        self.office = office
        self.capacity = capacity
        self.hiring_date = hiring_date
        self.enddate = enddate
        self.realcapacity = realcapacity

    def _set_enddate_for_update(self):
        if self.enddate:
            return ", enddate='{}'".format(self.enddate)
        else:
            return ''

    def _set_startdate_for_update(self):
        if self.hiring_date:
            return ", startdate='{}'".format(self.hiring_date)
        else:
            return ''

    def build_update(self):
        return """
            UPDATE `{BIGQUERY_EMPLOYEE_CAPACITY}` SET
                email='{email}',
                month='{month}',
                year='{year}',
                fte={fte},
                office='{office}',
                jira={jira},
                capacity={capacity},
                realcapacity={realcapacity}
                {hiring_date}
                {enddate}
            WHERE
                email='{email}' AND
                month='{month}' AND
                year='{year}'
        """.format(
            email=self.email,
            year=self.year,
            month=self.month,
            jira=self.jira,
            fte=self.fte,
            office=self.office,
            capacity=self.capacity,
            hiring_date=self._set_startdate_for_update(),
            realcapacity=self.realcapacity,
            enddate=self._set_enddate_for_update(),
            BIGQUERY_EMPLOYEE_CAPACITY=BIGQUERY_EMPLOYEE_CAPACITY,
        )

    def _get_insert_columns(self):
        basic_columns = 'email, month, year, fte, office, jira, capacity, realcapacity'  # noqa: E501

        if self.hiring_date:
            basic_columns += ', startdate'

        if self.enddate:
            basic_columns += ', enddate'

        return basic_columns

    def _set_enddate_insert(self):
        if self.enddate:
            return ",'{}'".format(self.enddate)
        return ''

    def _set_startdate_insert(self):
        if self.hiring_date:
            return ",'{}'".format(self.hiring_date)
        return ''

    def build_insert(self):
        return """
            INSERT INTO `{BIGQUERY_EMPLOYEE_CAPACITY}` ({columns}) VALUES (
                '{email}',
                '{month}',
                '{year}',
                {fte},
                '{office}',
                {jira},
                {capacity},
                {realcapacity}
                {hiring_date}
                {enddate}
            )
        """.format(
            columns=self._get_insert_columns(),
            email=self.email,
            year=self.year,
            month=self.month,
            jira=self.jira,
            fte=self.fte,
            office=self.office,
            capacity=self.capacity,
            realcapacity=self.realcapacity,
            hiring_date=self._set_startdate_insert(),
            enddate=self._set_enddate_insert(),
            BIGQUERY_EMPLOYEE_CAPACITY=BIGQUERY_EMPLOYEE_CAPACITY,
        )



from ms_imputedhours_core.employee.data.constants import EXCLUDE_ALERT_EMAILS


def should_exclude_employee(email, date, hired_date, end_date):
    is_email_excluded = email in EXCLUDE_ALERT_EMAILS

    is_personal_email = "@makingscience" not in email

    is_enddate_before_date = end_date and (end_date.year, end_date.month) < (
        date.year,
        date.month,
    )

    is_hired_date_after_date = hired_date and (
        hired_date.year,
        hired_date.month,
    ) > (date.year, date.month)

    return (
        is_email_excluded
        or is_enddate_before_date
        or is_hired_date_after_date
        or is_personal_email
    )



import os
from datetime import datetime

from gc_google_services_api.gsheet import GSheet

RANGE_NAME = os.getenv('GOOGLE_SHEET_AGREEMENT_RANGE', 'A2:B8000')
CREDENTIALS_BASE64 = os.getenv('GOOGLE_SHEET_CREDENTIALS', '')
SHEET_NAMES_TO_IGNORE = ['Summary']


class Agreements(object):
    def __init__(self, spreadsheet_id) -> None:
        self.spreadsheet_id = spreadsheet_id
        self.service = GSheet()

    def get_hours_by_month(self, month, year, sheet_name):
        try:
            values = self.service.read_gsheet(
                sheet_name, self.spreadsheet_id, RANGE_NAME
            )
        except Exception as e:
            print('[ERROR] - get_hours_by_month: ', e)
            values = {'values': []}

        day_hours_data = {'totalHours': 0.0, 'days': {}}
        for day_data in values['values']:
            day_obj = datetime.strptime(day_data[0], '%d/%m/%Y')

            if day_obj.month == month and day_obj.year == year:
                day_hours = '{}'.format(day_data[1]).replace(',', '.')
                day_hours_data['totalHours'] += float(day_hours)
                day_hours_data['days'][day_obj.day] = day_hours

        return day_hours_data

    def get_hours_by_range(self, from_date, to_date, sheet_name):
        try:
            values = self.service.read_gsheet(
                sheet_name, self.spreadsheet_id, RANGE_NAME
            )
        except Exception as e:
            print('[ERROR] - get_hours_by_range: ', e)
            values = {'values': []}

        day_hours_data = {'totalHours': 0.0, 'days': {}}

        for day_data in values['values']:
            day_obj = datetime.strptime(day_data[0], '%d/%m/%Y')
            is_date_after_from_date = (day_obj.month, day_obj.year) >= (
                from_date.month,
                from_date.year,
            )
            is_date_before_to_date = (day_obj.month, day_obj.year) <= (
                to_date.month,
                to_date.year,
            )

            if is_date_after_from_date and is_date_before_to_date:
                is_day_after_from_date = (day_obj.day, day_obj.month) >= (
                    from_date.day,
                    from_date.month,
                )
                is_dat_before_to_date = (day_obj.day, day_obj.month) <= (
                    to_date.day,
                    to_date.month,
                )

                if is_day_after_from_date and is_dat_before_to_date:
                    day_key = '{}-{}'.format(day_obj.day, day_obj.month)
                    day_hours = '{}'.format(day_data[1]).replace(',', '.')
                    day_hours_data['totalHours'] += float(day_hours)
                    day_hours_data['days'][day_key] = day_hours

        return day_hours_data

    def get_all_office_names(self):
        try:
            values = self.service.get_sheetnames(self.spreadsheet_id)
        except Exception as e:
            print('[ERROR] - get_all_agreements: ', e)
            values = {}

        sheet_names = []
        for sheet in values.get('sheets', []):
            sheet_name = sheet['properties']['title']
            if sheet_name not in SHEET_NAMES_TO_IGNORE:
                sheet_names.append(sheet_name)

        return sheet_names



from calendar import monthrange
from datetime import datetime, timedelta

HUNDRED_PERCENT = 100
DEFAULT_FTE = 1.0  # 100 %


def get_real_fte(successfactor, employeeFTE, date):
    """Calculate employee FTE.
    When we want to calculate the FTE of an employee, we do it for a specific
    month since this can change.

     Rules:
         - If the date we want to calculate the FTE is from the current
           month into the future, then we will take the FTE value returned
           by the Success factor Bigquery table.
         - If for the month we want to calculate the FTE is prior to
           the current one and it already has a value in the Capacity
           table, this means that it was already calculated and we will
           use that value.

     args:
        successfactor (dict): BigQuery results from the successfactor table.
        employeeFTE (dict): Bigquery results from the Capacity table.
        date (date): Month on which the FTE is being calculated

    Returns:
        float: Real FTE for a employee in specific month.
    """
    today = datetime.today()
    is_past_date = (date.year, date.month) <= (today.year, today.month)

    try:
        # Some employees has null as FTE.
        fte = successfactor['FTE'] / HUNDRED_PERCENT
    except (KeyError, TypeError):
        fte = DEFAULT_FTE

    if employeeFTE and is_past_date:
        fte = employeeFTE['fte']

    return fte


def calculate_real_capacity(
    agreement_hours,
    hiring_date,
    end_date,
    from_date,
    to_date,
    fte,
    calculate_range=False,
):
    if not hiring_date:
        return 0

    if not calculate_range:
        last_day_of_month = monthrange(from_date.year, from_date.month)[1]
        to_date = from_date.replace(day=last_day_of_month)

    if end_date and end_date <= from_date or hiring_date >= to_date:
        return 0

    if hiring_date > from_date:
        from_date = hiring_date

    if end_date and end_date < to_date:
        to_date = end_date

    if calculate_range:
        days = agreement_hours["days"]
        total_hours = 0
        current_date = from_date
        while current_date <= to_date:
            key = f"{current_date.day}-{current_date.month}"
            if key in days:
                total_hours += float(days[key])
            current_date += timedelta(days=1)
    else:
        total_hours = float(agreement_hours["totalHours"])

    real_capacity = total_hours * fte
    return real_capacity



import unittest
from datetime import datetime
from unittest.mock import patch

from ms_imputedhours_core.employee.data.helpers.alert import (
    should_exclude_employee,
)


class TestSuite(unittest.TestCase):
    def test_should_exclude_employee_should_return_false_when_employee_hired_date_is_lower_than_date(
        self,
    ):  # noqa: E501
        date = datetime.strptime('02/28/22 00:00:00', '%m/%d/%y %H:%M:%S')
        hired_date = datetime.strptime(
            '01/01/22 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        end_date = None
        expected_result = False

        is_exclude_employee = should_exclude_employee(
            email='test@makingscience.com',
            date=date,
            hired_date=hired_date,
            end_date=end_date,
        )

        self.assertEqual(is_exclude_employee, expected_result)

    def test_should_exclude_employee_should_return_true_when_employee_hired_date_is_greater_than_date(
        self,
    ):  # noqa: E501
        date = datetime.strptime('02/28/22 00:00:00', '%m/%d/%y %H:%M:%S')
        hired_date = datetime.strptime(
            '05/01/22 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        end_date = None
        expected_result = True

        is_exclude_employee = should_exclude_employee(
            email='test@makingscience.com',
            date=date,
            hired_date=hired_date,
            end_date=end_date,
        )

        self.assertEqual(is_exclude_employee, expected_result)

    def test_should_exclude_employee_should_return_false_when_employee_end_date_is_greater_than_date(
        self,
    ):  # noqa: E501
        date = datetime.strptime('02/28/22 00:00:00', '%m/%d/%y %H:%M:%S')
        hired_date = datetime.strptime(
            '01/01/21 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        end_date = datetime.strptime('05/15/22 00:00:00', '%m/%d/%y %H:%M:%S')
        expected_result = False

        is_exclude_employee = should_exclude_employee(
            email='test@makingscience.com',
            date=date,
            hired_date=hired_date,
            end_date=end_date,
        )

        self.assertEqual(is_exclude_employee, expected_result)

    def test_should_exclude_employee_should_return_true_when_employee_end_date_is_lower_than_date(
        self,
    ):  # noqa: E501
        date = datetime.strptime('02/28/22 00:00:00', '%m/%d/%y %H:%M:%S')
        hired_date = datetime.strptime(
            '01/01/21 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        end_date = datetime.strptime('01/15/22 00:00:00', '%m/%d/%y %H:%M:%S')
        expected_result = True

        is_exclude_employee = should_exclude_employee(
            email='test@makingscience.com',
            date=date,
            hired_date=hired_date,
            end_date=end_date,
        )

        self.assertEqual(is_exclude_employee, expected_result)

    @patch(
        'ms_imputedhours_core.employee.data.helpers.alert.EXCLUDE_ALERT_EMAILS',
        ['test@test.com'],
    )  # noqa: E501
    def test_should_exclude_employee_should_return_true_when_employee_email_is_excluded(
        self,
    ):  # noqa: E501
        date = datetime.strptime('02/28/22 00:00:00', '%m/%d/%y %H:%M:%S')
        hired_date = datetime.strptime(
            '01/01/22 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        end_date = None
        expected_result = True

        from ..alert import should_exclude_employee

        is_exclude_employee = should_exclude_employee(
            email='jonathan.rodriguez@makingscience.com',
            date=date,
            hired_date=hired_date,
            end_date=end_date,
        )

        self.assertEqual(is_exclude_employee, expected_result)

    @patch(
        'ms_imputedhours_core.employee.data.helpers.alert.EXCLUDE_ALERT_EMAILS',
        ['test@test.com'],
    )  # noqa: E501
    def test_should_exclude_employee_should_return_true_when_employee_email_not_from_ms(
        self,
    ):  # noqa: E501
        date = datetime.strptime('02/28/22 00:00:00', '%m/%d/%y %H:%M:%S')
        hired_date = datetime.strptime(
            '01/01/21 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        end_date = datetime.strptime('05/15/22 00:00:00', '%m/%d/%y %H:%M:%S')
        expected_result = True

        is_exclude_employee = should_exclude_employee(
            email='test@test.com',
            date=date,
            hired_date=hired_date,
            end_date=end_date,
        )

        self.assertEqual(is_exclude_employee, expected_result)
